#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include <stdio.h>
#include <string.h>

#include "buzzer.h"
#include "cds.h"
#include "fnd.h"
#include "key.h"
#include "lcd.h"
#include "led.h"


#define EEP_FLAG	(unsigned char *)0x01
#define EEP_TEMP	(unsigned char *)0x02
#define EEP_DOOR	(unsigned char *)0x03


enum {
	NORMAL_MODE,
	POWER_MODE
};

enum {
	CLOSE,
	OPEN
};

extern volatile char key_flag;

volatile char disp_flag;
volatile unsigned char mode;
volatile unsigned char cur_temp, set_temp;
volatile unsigned char door_count, door_state, door_time;
volatile unsigned int door_tick;
volatile unsigned char set_lux, max_lux, min_lux, cur_lux;
volatile unsigned char led_bright;
volatile unsigned char down_flag, down_tick, down_time;


void timer0_init(void)
{
	TCCR0 = 0x05;
	TCNT0 = 256 - 125;
	TIMSK |= 0x01;
}

ISR(TIMER0_OVF_vect)
{
	static unsigned char tick0;

	TCNT0 = 256 - 125;

	tick0++;

	if( tick0 < led_bright )				LED_OUT &= ~LED1;
	else if( tick0 >= led_bright && tick0 < 10 )	LED_OUT |= LED1;
	else if( tick0 == 10 )					tick0 = 0;

	if( door_state ) {
		door_tick++;
		////////////////////////////////////////////////////////////////////
		// [ 4]    ִ   Ʒ  ۵ǵ α׷
		//		Ͻÿ.
		////////////////////////////////////////////////////////////////////
		if( (door_tick % 500) == 0 ) {
			if( door_time < 30 )		BUZZER_OUT |= 1 << BUZZER;

			if( cur_temp < 14 )	disp_flag ^= 0x01;
		}

		if( (door_tick % 1000) == 0 ) {
			if( door_time < 99 )	door_time++;

			if( door_time >= 11 && door_time <= 20 ) {
				if( door_time % 2 )	BUZZER_OUT &= ~(1 << BUZZER);
			}
			else if( door_time >= 21 )	BUZZER_OUT &= ~(1 << BUZZER);
		}
		
		if( door_tick == 2000 ) {
			door_tick = 0;
			if( cur_temp < 14 )	cur_temp++;
			else {
				disp_flag = 1;
				TIMSK |= 0x40;
			}
		}
	}

}

void timer1_init(void)
{
	TCCR1A = 0x00;
	TCCR1B = 0x03;
	TCCR1C = 0x00;
	TCNT1 = 65536 - 25000;
	TIMSK |= 0x04;
}

ISR(TIMER1_OVF_vect)
{
	TCNT1 = 65536 - 25000;

	if( (cur_temp != set_temp) && !door_state ) {
		if( !down_flag ) {
			down_flag = 1;
			down_tick = 0;
		}
		else {
			down_tick++;
			if( down_tick % 5 == 0 )	disp_flag ^= 0x01;
			
			if( down_tick >= down_time ) {
				down_tick = 0;
				if( cur_temp > set_temp )		cur_temp--;
				else if( cur_temp < set_temp )	cur_temp++;
			}
		}
	}
	else if( (cur_temp == set_temp) && !door_state ) {
		disp_flag = 1;
		TIMSK |= 0x40;
	}
}

void mcu_init(void)
{
	buzzer_init();
	cds_init();
	fnd_init();
	key_init();
	led_init();
	lcd_init();
}

void variable_init(void)
{
	if( eeprom_read_byte(EEP_FLAG) == 107 ) {
		set_temp = eeprom_read_byte(EEP_TEMP);
		door_count = eeprom_read_byte(EEP_DOOR);
	}
	else {
		set_temp = 4;
		door_count = 0;
		eeprom_write_byte(EEP_FLAG, 107);
		eeprom_write_byte(EEP_TEMP, 4);
		eeprom_write_byte(EEP_DOOR, 0);
	}

	disp_flag = 0;

	mode = NORMAL_MODE;

	cur_temp = 12;

	set_lux = 0;
	max_lux = 99;
	min_lux = 0;
	cur_lux = 0;

	down_time = 20;

	door_state = CLOSE;
}

void start_refregerator(void)
{
	lcd_gotoxy(0, 0);
	lcd_string("  Refrigerator  ");
	lcd_gotoxy(0, 1);
	lcd_string("     System     ");

	for( int i = 0; i < 3; i++ ) {
		SEG_OUT = 0xC0;
		DIG_OUT = 0x04;
		LED_OUT = 0x00;
		_delay_ms(500);

		SEG_OUT = 0xFF;
		DIG_OUT = 0x07;
		LED_OUT = 0x0F;
		_delay_ms(500);
	}

	disp_flag = 1;
}

////////////////////////////////////////////////////////////////////////////////
// [ 1] LCD CGRAM ̿Ͽ LCD µ Ÿ ɡ ơ ǥ
//		ǵ α׷ Ͻÿ.
////////////////////////////////////////////////////////////////////////////////
void lcd_cgram(void)
{
	unsigned char cgram[8] = { 0x06, 0x09, 0x09, 0x06, 0x00, 0x00, 0x00, 0x00 };

	lcd_command_write(0x40);
	for( int i = 0; i < 8; i++ )	lcd_data_write(cgram[i]);
}

void init_screen(void)
{
	lcd_gotoxy(0, 0);
	printf("Mode:Nor  Lux:%02d", cur_lux);
	lcd_gotoxy(0, 1);
	printf("Temp:%02d C Door:C", set_temp);
	lcd_gotoxy(7, 1);
	lcd_data_write(0x00);
}

void check_condition(void)
{
	if( mode == NORMAL_MODE ) {
		if( cur_temp != set_temp )	led_light(LED2 | LED3);
		else					led_light(LED2);
	}
	else {
		if( cur_temp == set_temp ) {
			down_time = 20;

			lcd_gotoxy(0, 0);
			lcd_string("Mode:Nor  Lux:00");

			LED_OUT |= LED4;

			mode = NORMAL_MODE;
		}
	}

	if( set_lux && !door_state ) {
		cur_lux = (unsigned char)(cds_read() / 10.23);

		lcd_gotoxy(14, 0);
		printf("%02d", cur_lux);

		////////////////////////////////////////////////////////////////////
		// [ 2] LED1"  Ʒ  ۵ǵ α׷ Ͻÿ.
		//		(Ÿ̸ ̿)
		////////////////////////////////////////////////////////////////////
		if( cur_lux >= 0 && cur_lux <= 20 )		led_bright = 9;
		else if( cur_lux >= 21 && cur_lux <= 39 )	led_bright = 7;
		else if( cur_lux >= 40 && cur_lux <= 59 )	led_bright = 5;
		else if( cur_lux >= 60 && cur_lux <= 79 )	led_bright = 3;
		else if( cur_lux >= 80 && cur_lux <= 99 )	led_bright = 0;
	}

	if( door_state ) {
		lcd_gotoxy(6, 1);
		printf("%02d", door_time);
	}
}

void set_temperature(void)
{
	lcd_gotoxy(6, 1);
	printf("%02d", set_temp);
}

void set_luxlight(void)
{
	lcd_gotoxy(5, 1);
	printf("%02d", max_lux);
	lcd_gotoxy(13, 1);
	printf("%02d", min_lux);
}

unsigned char set_mode(unsigned char keyin)
{
	char key = keyin;
	char set_mode = 0, lux, lux_flag = 0;
	enum {
		NONE,
		SET_TEMP,
		SET_LUX
	};

	lcd_gotoxy(0, 0);
	lcd_string("    Set_Mode    ");
	lcd_gotoxy(0, 1);
	lcd_string("Press Key SW1-2 ");

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			switch( key ) {
				case KEY_POWER :
					if( set_mode == NONE ) {
						lcd_gotoxy(0, 0);
						lcd_string("Set Temperature ");
						lcd_gotoxy(0, 1);
						printf("      %02d C      ", set_temp);
						lcd_gotoxy(8, 1);
						lcd_data_write(0);

						set_mode = SET_TEMP;
						TIMSK &= ~0x04;
					}
					break;

				case KEY_DOOR :
					if( set_mode == NONE ) {
						lcd_gotoxy(0, 0);
						lcd_string(" Set Light Lux  ");
						lcd_gotoxy(0, 1);
						printf(" Max:%02d  Min:%02d ", max_lux, min_lux);

						set_mode = SET_LUX;
					}
					break;

				case KEY_UP :
					if( set_mode == SET_TEMP ) {
						if( set_temp < 10 )	set_temp++;
						else	buzzer_on();
					}
					else if( set_mode == SET_LUX ) {
						if( lux_flag == 99 )	lux_flag = 0;
						else				lux_flag = 99;
					}
					break;

				case KEY_DOWN :
					if( set_mode == SET_TEMP ) {
						if( set_temp > 0 )	set_temp--;
						else	buzzer_on();
					}
					else if( set_mode == SET_LUX ) {
						if( lux_flag == 1 )	lux_flag = 0;
						else				lux_flag = 1;
					}
					break;

				case KEY_SET :
					if( set_mode == SET_TEMP ) {
						TIMSK |= 0x04;
						eeprom_write_byte(EEP_FLAG, 107);
						eeprom_write_byte(EEP_TEMP, set_temp);
					}
					else if( set_mode == SET_LUX )	set_lux = 1;
					set_mode = 0;
					return key;
			}
		}

		if( set_mode == SET_TEMP )	set_temperature();
		else if( set_mode == SET_LUX ) {
			lux = (unsigned char)(cds_read() / 10.23);
			if( lux_flag == 99 )		max_lux = lux;
			else if( lux_flag == 1 )	min_lux = lux;
			set_luxlight();
		}

		if( cur_temp != set_temp )	led_light(LED2 | LED3);
		else					led_light(LED2);
	}
}

void door_condition(void)
{
	door_state ^= 0x01;

	if( door_state == OPEN ) {
		if( door_count < 99 )	door_count++;
		eeprom_write_byte(EEP_FLAG, 107);
		eeprom_write_byte(EEP_DOOR, door_count);

		lcd_gotoxy(0, 0);
		lcd_string("   Door Open    ");
		lcd_gotoxy(0, 1);
		printf(" Time:00 Use:%02d ", door_count);

		door_time = 0;
		door_tick = 0;
	}
	else {
		BUZZER_OUT |= 1 << BUZZER;
		init_screen();
	}
}

void power_mode(void)
{
	if( cur_temp <= set_temp ) {
		for( int i = 0; i < 3; i++ ) {
			lcd_gotoxy(0, 0);
			lcd_string(" Not Power Mode ");
			lcd_gotoxy(0, 1);
			lcd_string("Please Set Temp!");

			buzzer_on();

			lcd_command_write(0x01);
			_delay_ms(500);
		}

		init_screen();
	}
	else {
		////////////////////////////////////////////////////////////////////
		// [ 3]
		// ()  µ  µ ū    ۵ǵ α׷
		//	Ͻÿ.
		////////////////////////////////////////////////////////////////////
		lcd_gotoxy(0, 0);
		lcd_string("Mode:Pow  Lux:00");
		lcd_gotoxy(0, 1);
		printf("Temp:%02d C Door:C", set_temp);
		lcd_gotoxy(7, 1);
		lcd_data_write(0x00);

		down_time = 10;
		led_light(LED4);

		mode = POWER_MODE;
	}
}

int main(void)
{
	unsigned char key = 0;

	mcu_init();
	variable_init();
	start_refregerator();

	sei();

	timer0_init();
	timer1_init();
	fdevopen((void *)lcd_data_write, 0);

	lcd_cgram();
	init_screen();

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			switch( key ) {
				case KEY_POWER :
					power_mode();
					break;

				case KEY_DOOR :
					door_condition();
					break;

				case KEY_SET :
					key = set_mode(key);
					init_screen();
					break;
			}
		}

		check_condition();
	}

	return 0;
}
